
#include "stbcfg.h"
#include <common.h>
#include "tlv_image.h"


/**********************/
/* Local Definitions  */
/**********************/

#define DECARM_VECTOR_TBL_SZ       0x40 /* 8*32, 8 vectors, each 32 bytes and 8 labels*/
#define DECARM_CODE_START_ADDR     ARM9_RESET_ADDR + 0x1000
#define DECARM_SHARED_RAM_ADDR     ARM9_RESET_ADDR + 0x1004
/*Legacy addresses to ensure backward compatibilty of ARM9 ucode */
#define DECARM_CODE_START_ADDR_LEG     ARM9_RESET_ADDR + 0x40
#define DECARM_SHARED_RAM_ADDR_LEG     ARM9_RESET_ADDR + 0x44
#define ARM_INSTR_BRANCH_TO_ABSOLUTE_FROM_OFFSET_8 0xE59FF000


/* defined in cpu/arm1176/start.S */
extern void decarm_spin_begin(void);
extern void decarm_spin_end(void);

#if (INSTR_MEM_VALID_MODE == USE_IMV)
void internal_arm926_config_imv();
#endif /*(INSTR_MEM_VALID_MODE == USE_IMV)*/

struct tag_decarm decarm_tag = {0, 0, 0, 0}; /* will be populate once boot_decarm is called */

void boot_decarm (void);

/* local definitions*/
static void bring_decarm_out_of_reset(void);

#if VIDEO_DRIVER_LOADS_ARM9 == YES
bool internal_decarm_release_and_spin(void);
#else
static void decarm_load_and_boot(void);
static bool write_word_to_926_address_space(u_int32 uAddr, u_int32 uValue);
static bool write_arm926_reset_vector_jump(u_int32 uTargetAddr);
#endif

/*
 * boot_decarm will load decarm image from flash if needed, and take decarm out of reset.
 */
void boot_decarm (void)
{
   #if VIDEO_DRIVER_LOADS_ARM9 == YES
   /* Bring it out of reset and spin */
   internal_decarm_release_and_spin();
   #else
   /* load arm slave image from ROM and bring it out of reset */
   decarm_load_and_boot();
   #endif
}

/********************************************************************/
/*  bring_decarm_out_of_reset                                       */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      None                                                        */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Ensure that the ARM926 address remapping is set up so that  */
/*      it boots from the memory location we see as 0x10000 then    */
/*      bring it out of reset.                                      */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Nothing                                                     */
/********************************************************************/
static void bring_decarm_out_of_reset(void)
{
  
   /* Set the ARM 9 REMAP bit, so that it address space view shifts to the second 64K boundary */
   CNXT_SET( RST_DECARM_CTL_REG,
             RST_DECARM_CTL_REMAP_MASK,
             RST_DECARM_CTL_REMAP_ACTIVE);
             
   /* Bring ARM 9 out of reset ! */
   CNXT_SET( RST_DECARM_CTL_REG,
             RST_DECARM_CTL_RESET_MASK,
             RST_DECARM_CTL_REMAP_RELEASE_RESET);
}


#if VIDEO_DRIVER_LOADS_ARM9 == NO
/********************************************************************/
/*  write_word_to_926_address_space                                 */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      uAddr   - Physical address of the location to write.        */
/*      uValue  - Value to write to *pAddr.                         */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Write a word to the physical address (as seen by the ARM    */
/*      1176) in such a way that we are sure it will have been      */
/*      flushed to physical RAM and, hence, visible to the ARM926.  */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Hangs on failure, returns TRUE on success (currently)       */
/********************************************************************/
static bool write_word_to_926_address_space(u_int32 uAddr, u_int32 uValue)
{
   u_int32 uDummy;
   
   do{
      /* Write a "b waithere" instruction to the ARM926 reset vector address */
      *(LPREG)uAddr = uValue;
   
      /* Read it back to make sure write to physical RAM completed */
      uDummy = *((LPREG)uAddr);
      
      if(uDummy != uValue)
      {
         printf("Unable to write word to ARM926 address space!!\n");
      }
   } while (uDummy != uValue);
   
   return(TRUE);
}

/********************************************************************/
/*  write_arm926_reset_vector_jump                                  */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      uTargetAddr - The physical address of the instruction that  */
/*                    the ARM926 should jump to when it is taken    */
/*                    out of reset.                                 */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Write the ARM926 reset vector with an instruction that will */
/*      cause a jump to a particular location in memory when the    */
/*      CPU is taken out of reset.                                  */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Hangs on failure, returns TRUE on success (currently)       */
/********************************************************************/
static bool write_arm926_reset_vector_jump(u_int32 uTargetAddr)
{
   bool bRetcode;
   
   /*printf("Writing jump instruction to ARM926 reset vector...\n");*/
   
   bRetcode = write_word_to_926_address_space(ARM9_RESET_ADDR+8,
                                              uTargetAddr);
                                              
   if(bRetcode == TRUE)
   {
      bRetcode = write_word_to_926_address_space(
                                    ARM9_RESET_ADDR,
                                    ARM_INSTR_BRANCH_TO_ABSOLUTE_FROM_OFFSET_8);
   }
   return(bRetcode);
}

static void decarm_load_and_boot(void)
{
   u_int32         *puSlaveImageBase  = 0;

   /* find_and_load_tlv_image will populate decarm_tag structure for ataglist */
   if (find_and_load_tlv_image( IMAGE_TYPE_UNC_SLAVE, &puSlaveImageBase) == TRUE)
   {
      printf("Decarm: Load\n");
   }
   else
   {
      printf("DECARM image is not found in flash, DECARM remains reset\n");
      return;
   }

   /* boot the image */
   if( puSlaveImageBase == NULL )
   {
      printf("No DECARM image entry. ERROR!!!\n");
      return;
   }

   /* Produce a dummy reset vector to branch to slave entry address. */
   write_arm926_reset_vector_jump((u_int32)puSlaveImageBase);

   /* program the shardRAM start and Runtime start to DECARM_SHARED_RAM_ADDR and DECARM_SHARED_RAM_ADDR */
   CNXT_SET( DECARM_SHARED_RAM_ADDR, 0xFFFFFFFF, decarm_tag.sharedram_start );
   CNXT_SET( DECARM_CODE_START_ADDR, 0xFFFFFFFF, decarm_tag.code_start );

   /* Add an identifier(0xEEEEEEEE) at ARM9_CODE_START_ADDR_LEG (old code ram 
    * address) to help video SW/FW identify that this codeldr is new one and 
    * ARM9_CODE_START_ADDR and ARM9_SHARED_RAM_ADDR has shifted to new offset
    * 0x1000 and 0x1004 
    */
   CNXT_SET( DECARM_CODE_START_ADDR_LEG, 0xFFFFFFFF, 0xEEEEEEEE );

   /* take decarm out of reset */
   bring_decarm_out_of_reset();

   return;


} /* decarm_load_and_boot */
#endif /*  VIDEO_DRIVER_LOADS_ARM9 == NO */

#if VIDEO_DRIVER_LOADS_ARM9 == YES
/********************************************************************/
/*  internal_decarm_release_and_spin                        */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      None                                                        */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Take ARM926 out of reset and spin                           */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Hangs on failure, returns TRUE on success (currently)       */
/********************************************************************/
bool internal_decarm_release_and_spin(void)
{
   bool  bRetcode = TRUE;
   LPREG pDest = (LPREG)ARM9_RESET_ADDR;
   LPREG pSrc  = (LPREG)decarm_spin_begin;
   
   printf("Decarm: spin\n");

   CNXT_SET( DECARM_SHARED_RAM_ADDR, 0xFFFFFFFF, 0 );
   CNXT_SET( DECARM_CODE_START_ADDR, 0xFFFFFFFF, 0 );

   /* In this state, the ARM926 is taken out of reset, then ARM926 waits for
   the video driver to write words in the DECARM_SHARED_RAM_ADDR and DECARM_CODE_START_ADDR.
   When ARM926 sees non-zero values in these two addresses, ARM926
   jumps to the address specified in DECARM_CODE_START_ADDR. */
   while (pSrc < (LPREG)decarm_spin_end)
   {
       *pDest++ = *pSrc++;
   }

   while( pDest < (LPREG)DECARM_CODE_START_ADDR)
   {
      *pDest++ = 0x0;
   }
  
    /* Add an identifier(0xEEEEEEEE) at ARM9_CODE_START_ADDR_LEG (old code ram 
    * address) to help video SW/FW identify that this codeldr is new one and 
    * ARM9_CODE_START_ADDR and ARM9_SHARED_RAM_ADDR has shifted to new offset
    * 0x1000 and 0x1004 */
   CNXT_SET( DECARM_CODE_START_ADDR_LEG, 0xFFFFFFFF, 0xEEEEEEEE );

   #if (INSTR_MEM_VALID_MODE == USE_IMV)
   /* Enable Instruction Memory Validation on ARM9 spin instructions */
   internal_arm926_config_imv();
   #endif /*(INSTR_MEM_VALID_MODE == USE_IMV)*/

   bring_decarm_out_of_reset();
   return bRetcode;
}
#endif
