 /****************************************************************************/ 
 /*                   NXP PROPRIETARY AND CONFIDENTIAL                       */
 /*                       SOFTWARE FILE/MODULE HEADER                        */
 /*                    NXP Semiconductors Inc. (c) 2008                      */
 /*                         All Rights Reserved                              */
 /****************************************************************************/
 /*
  * Filename:    ram.c
  *
  *
  * Description: RAM routines.
  *
  *
  * Author:      Angela Swartz
  *
  ****************************************************************************/
 /* $Id: ramc.c 55536 2008-09-12 17:08:25Z swartzwg $
  ****************************************************************************/ 

/******************/
/* Include Files  */
/******************/
#include "stbcfg.h"
#include "cnxt_types.h"

#define MARK_HERE printf("Debug line %d\n", __LINE__ )

#define SIZE_IDX  0
#define START_IDX 1

/*************************/
/* Internal Definitions  */
/*************************/
static u_int32 GetMCRAMSize( u_int8 uMC ); 

bool get_meminfo( u_int32 *pMemInfo, u_int32 *puNumOfEntries );
int GetRAMSize(void);
bool GetTopRam(u_int32 *puTopRamAddr);
/********************************************************************/
/*  GetRAMSize                                                      */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Query the RAM controller registers to find out how much     */
/*      RAM is installed.                                           */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      None.                                                       */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Size of physical RAM in bytes.                              */
/********************************************************************/
int GetRAMSize(void)
{
   static u_int32 guRAMSize = 0;

   if( guRAMSize == 0 )
   {
      guRAMSize = GetMCRAMSize(0) + GetMCRAMSize(1);

      #if (OVERRIDE_ACTUAL_TARGET_RAM_SIZE == YES)
      /* If the RAM size reported is to be overridden, make sure that
       * the override is sensible before returning the config file value
       */
      if( (RAM_SIZE_OVERRIDE * 1024) < guRAMSize )
      {
         guRAMSize = RAM_SIZE_OVERRIDE * 1024;
      }
      #endif

   }
   return( (int)guRAMSize );
}


/********************************************************************/
/*  GetMCRAMSize                                                    */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Query the RAM controller registers to find out how much     */
/*      RAM is installed on the specified MC controller on a IRD    */
/*      containing a Pecos B (or later)                             */
/*      chip.                                                       */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      uMC - MC unit number                                        */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Size of physical RAM in bytes.                              */
/********************************************************************/
static u_int32 GetMCRAMSize(u_int8 uMC)
{
   u_int32 uRAMSizePerMC;
   u_int32 uColBits = 0;
   u_int32 uRowBits = 0;
   u_int32 uBanks   = 0;
   u_int32 uWork;
   u_int32 uRAMWidth; 
   
   /* look throught the MC_CFG0_M0 and MC_CFG0_M1 to find out how much RAM are there. */
   if( uMC == 0 )
   {
      uWork = CNXT_GET(MC_CFG0_REG, (MC_CFG0_COL_BITS_MASK|MC_CFG0_BANK_BITS_MASK|MC_CFG0_ROW_BITS_MASK));
      uRAMWidth = MC0_WIDTH;
      if( (MC0_WIDTH == 32) && (ISNEVIS))
      {
         /* NEVIS M0 data bus can not be 32bits, it has to 16bits */
         uRAMWidth = 16;
      }
   }
   else 
   {
      uWork = CNXT_GET(MC_CFG0_M1_REG, (MC_CFG0_COL_BITS_MASK|MC_CFG0_BANK_BITS_MASK|MC_CFG0_ROW_BITS_MASK));
      uRAMWidth = MC1_WIDTH;
   }

   /* Determine the number of column bits */
   switch(uWork & MC_CFG0_COL_BITS_MASK)
   {
      case MC_CFG0_COL_BITS_VALUE_9:   uColBits = 9;     break;
      case MC_CFG0_COL_BITS_VALUE_10:  uColBits = 10;    break;
      case MC_CFG0_COL_BITS_VALUE_11:  uColBits = 11;    break;
      default:  return 0; /* Undefined value - force an error */
   }
   
   /* Determine the number of row bits */
   switch(uWork & MC_CFG0_ROW_BITS_MASK)
   {
      case MC_CFG0_ROW_BITS_VALUE_12:    uRowBits = 12;    break;
      case MC_CFG0_ROW_BITS_VALUE_13:    uRowBits = 13;    break;
      case MC_CFG0_ROW_BITS_VALUE_14:    uRowBits = 14;    break;
      default:  return 0; /* Undefined value - force an error */
   }
   
   /* Determine the number of banks */
   switch(uWork & MC_CFG0_BANK_BITS_MASK)
   {
      case MC_CFG0_BANK_BITS_VALUE_2:    uBanks = 2;       break;
      case MC_CFG0_BANK_BITS_VALUE_3:    uBanks = 3;       break;
      default:  return 0; /* Undefined value - force an error */
   }
   
   /* Having worked out the memory arrangement, calculate the size per devices */
   uRAMSizePerMC = (1 << (uRowBits + uColBits + uBanks + 1));
   uRAMSizePerMC = (uRAMSizePerMC * (uRAMWidth/16));
   
   return(uRAMSizePerMC);
}
/********************************************************************/
/*  GetRAMWidth                                                     */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Determine the width of the memory bus.                      */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      None.                                                       */
/*                                                                  */
/*  RETURNS:                                                        */
/*      Width of the memory bus.                                    */
/********************************************************************/
int GetRAMWidth()
{
   u_int32 uRAMWidth = 0;

   uRAMWidth = MC0_WIDTH + MC1_WIDTH;

   return (uRAMWidth);
}

/* get the memory information, the first parm is array of u_int32,
 * the size of the array must be at least 2, best to be 4 in case 
 * there are 2 entries for the meminfo(each meminfo has 2 words). 
 * puNumOfEntries should be set to sizeof(pMemInfo)/4.  
 */
bool get_meminfo( u_int32 *pMemInfo, u_int32 *puNumOfEntries )
{
   bool    bDS            = FALSE;
   bool    bBankedMemory  = FALSE;
   u_int32 uMC_Size[2];
   u_int32 uMC_Width[2];
   u_int32 uDensity[2];
   u_int32 uMCSizeTotal;
   u_int32 uMaxDensity;
   u_int32 uTemp;
   u_int8  uLoop;
   u_int8  uBankedMemoryTM[]={ 16, 17, 18, 22, 24, 25, 26, 28, 29, 30, 36, 
                               41, 42, 43, 44, 45, 46, 47, 52, 53, 56, 57, 
                               58, 60, 61, 62 };
   static  u_int32  gMemInfo[2][2];
   static  u_int32  guMemInfoCount = 0;
   
   if( guMemInfoCount == 0 )
   {
      /* find out whether Device Swap is enable or not */
      bDS = (bool)((CNXT_GET(MC_TILE_CFG_REG,  MC_TILE_CFG_DEV_SWAP_MASK))>>MC_TILE_CFG_DEV_SWAP_SHIFT);
   
      /* find out whether the tile mode in use is banked or unified. */ 
      uTemp = ((CNXT_GET(MC_TILE_CFG_REG,  MC_TILE_CFG_TILE_CFG_MASK))>>MC_TILE_CFG_TILE_CFG_SHIFT);
      for( uLoop = 0; uLoop<sizeof(uBankedMemoryTM); uLoop++ )
      {
         if( uTemp == uBankedMemoryTM[uLoop] )
         {
            /* found a match, it is banked memory configuration */
            bBankedMemory = TRUE;
            break;
         }
      }
   
      for( uTemp = 0; uTemp < 2; uTemp++ )
      {
         uMC_Size[uTemp] = GetMCRAMSize(uTemp);
         uMC_Width[uTemp] = ((uTemp==0)?MC0_WIDTH:MC1_WIDTH);
         uDensity[uTemp] = uMC_Size[uTemp];
         /* density per interfaces is 2x RAM size if the MC data bus
            width is 16bits for banked memory configuration */
         if( (bBankedMemory == TRUE) && (uMC_Width[uTemp] == 16) )
         {
            uDensity[uTemp] += uDensity[uTemp];
         }
   
      }
      /* find out the largest density between the two memroy interfaces */
      uMaxDensity = (uDensity[0] > uDensity[1])?uDensity[0]:uDensity[1]; 
   
      /* total memory size */
      uMCSizeTotal = uMC_Size[0] + uMC_Size[1];
   
      /* check if total ram size is forced by software config option */
      #if (OVERRIDE_ACTUAL_TARGET_RAM_SIZE == YES)
      /* If the RAM size reported is to be overridden, make sure that
       * the override is sensible before returning the config file value
       */
      if( (RAM_SIZE_OVERRIDE * 1024) < uMCSizeTotal )
      {
         uMCSizeTotal = RAM_SIZE_OVERRIDE * 1024;
      }
      #endif
   
      /* find out whether there will be alias in the memory map. The rule is if the first MC size is greater than the max density,
         then there will be an alias/hole in the middle of memory map. Otherwise, there is none. Note, if there is an alias/hole on
         the top of RAM, we ignore it, as MEMINFO in ataglist will stop at total RAM size. */
      if( bBankedMemory == TRUE )
      {
         gMemInfo[0][START_IDX] = 0;
         gMemInfo[0][SIZE_IDX]  = (bDS)?uMC_Size[1]:uMC_Size[0];
         guMemInfoCount = 1;
         if( gMemInfo[0][SIZE_IDX] >= uMCSizeTotal )
         {
            /* memory override, so reduce the mem size */
            gMemInfo[0][SIZE_IDX] = uMCSizeTotal;
         }
         else /* need another meminfo entry */
         {
            gMemInfo[1][START_IDX] = uMaxDensity;
            /* MC 0 will be on the top if Device swap is enable, MC 1 will be on the top if Device swap is disabled */
            gMemInfo[1][SIZE_IDX]  = (bDS)?uMC_Size[0]:uMC_Size[1];
            guMemInfoCount++;
   
            /* make sure the size is within the RAM_OVERRIDE limit */
            if( gMemInfo[1][SIZE_IDX] > (uMCSizeTotal - gMemInfo[0][SIZE_IDX]) )
            {
               gMemInfo[1][SIZE_IDX] = (uMCSizeTotal - gMemInfo[0][SIZE_IDX]);
            }
         }
      }
      else  /* unified memory configuation, there is only one meminfo entry */
      {
         gMemInfo[0][START_IDX] = 0;
         gMemInfo[0][SIZE_IDX]  = uMCSizeTotal;
         guMemInfoCount = 1;
      }
   
      /* If the config option OVERRIDE_PHYS_RAM_BASE is set, ram base
         address will be moved up. This is useful only when there is alias in 
         memory, so that alias memory will be used to make the whole memory 
         map continous */
      #if( OVERRIDE_PHYS_RAM_BASE == YES )
      if( guMemInfoCount > 1 )
      {
         gMemInfo[0][START_IDX] = RAM_BASE_OVERRIDE;
         gMemInfo[0][SIZE_IDX]  += gMemInfo[0][START_IDX];
      }
      #endif
   }

   /* the MemInfo is already populated now, return the global data */
   pMemInfo[0] = gMemInfo[0][SIZE_IDX];
   pMemInfo[1] = gMemInfo[0][START_IDX];

   if( *puNumOfEntries <= 1 && guMemInfoCount > 1  )
   {
      /* not enough space to hold the meminfo entries, return false */
      *puNumOfEntries = 1;
      return FALSE;
   }
   if( *puNumOfEntries > 1 && guMemInfoCount > 1 )
   {
      pMemInfo[2] = gMemInfo[1][SIZE_IDX];
      pMemInfo[3] = gMemInfo[1][START_IDX];
   }
   *puNumOfEntries = guMemInfoCount;

   return TRUE;
}

/* return the top ram address */
bool GetTopRam(u_int32 *puTopRamAddr)
{
   static u_int32 guTopRam = 0;
   u_int32        MemInfo[4];
   u_int32        uCount = 2;
   u_int8         uIndex;

   /* check NULL pointer */
   if( puTopRamAddr == NULL )
   {
      return FALSE;
   }

   if( guTopRam == 0 )
   {
      /* first time it is called */
      get_meminfo( MemInfo, &uCount );
      uIndex = (uCount == 1)?0:2;
      /* index 0,    1,     2,    3 
       *     size, start, size, start
       *
       *              start( 1, or 3) + size( 0, or 2)   
       */
      guTopRam = MemInfo[uIndex+1] + MemInfo[uIndex];
   }

   /* return the top ram address to caller */
   *puTopRamAddr = guTopRam;

   return TRUE;
}


