/*
 * (C) Copyright 2008 Angela Swartz ,
 * angela.swartz@nxp.com, NXP semiconductors.
 *
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

/******************/
/* Include Files  */
/******************/
#include "stbcfg.h"

#include <common.h>

#ifndef CONFIG_CMD_NFC

/**********************/
/* Local Definitions  */
/**********************/
static void CalcClkFreqAndPeriodForIOPLL (ulong *frequency,
                                          ulong *period,
                                          ulong xtal_freq );

/***************/
/* Global Data */
/***************/

/*****************/
/* External Data */
/*****************/
/* defined in cfi_flash.c */
extern flash_info_t flash_info[CFG_MAX_FLASH_BANKS_DETECT];	/* FLASH chips info */

/* setup ROM descriptors for primary and secondary control registers */
void rom_early_init(void)
{

   ulong value, mask;
   LPREG pPrimary, pSecondary;
   ulong uMaxDiv;

   ulong freq;
   ulong IOClkPeriod32;
   ulong uMaxReadAccess, data, val;
   u8    bank;

   /*******************************************/
   /* Calculate Memory & I/O bus clock period */
   /*******************************************/
   CalcClkFreqAndPeriodForIOPLL (&freq, &IOClkPeriod32, CRYSTAL_FREQUENCY);
   
   /*
    * Set the PCI sync bits
    */
   value = (( 2UL << PCI_ROM_REQ0_SYNC_SHIFT ) |
            ( 2UL << PCI_ROM_REQ1_SYNC_SHIFT ) |
            ( 2UL << PCI_ROM_REQ2_SYNC_SHIFT ));
   mask = (PCI_ROM_REQ0_SYNC_MASK | 
           PCI_ROM_REQ1_SYNC_MASK | 
           PCI_ROM_REQ2_SYNC_MASK); 
   CNXT_SET(PCI_ROM_MODE_REG, mask, value);

   /****************************************************/
   /* Determine the maximum read access time required. */
   /****************************************************/
   uMaxReadAccess = 126; /* this value should be configurable via config option */
  
   data = uMaxReadAccess * 100;
   val = uMaxDiv = 0;
   while(val < data)
   {
      ++uMaxDiv;
      val += IOClkPeriod32;
   }

   for (bank = 0; bank < CFG_MAX_FLASH_BANKS_DETECT; bank++)
   {
       pPrimary   = (LPREG) PCI_ROM_DESC_BASE + bank;
       pSecondary = (LPREG) PCI_EXT_DESC_BASE + bank;

       /* Set default values to guarantee access. */
       *pPrimary = (PCI_ROM_DESC_WIDTH_16                                |
                    (bank << PCI_ROM_DESC_CS_SHIFT)                      |
                    (15 << PCI_ROM_DESC_BURST_WAIT_SHIFT)                |
                    (uMaxDiv << PCI_ROM_DESC_READ_WAIT_SHIFT)            |
                    (uMaxDiv << PCI_ROM_DESC_WRITE_WAIT_SHIFT)           |
                    PCI_ROM_DESC_TYPE_ROM                                |
                    PCI_ROM_DESC_BURST_DISABLE);
       *pSecondary = 0x0;
  }
}


/* initialize the ROM MAP register in order to access ROM devices */
void rom_map_init(u8 bank, u32 bankbase, u32 banksize)
{

   LPREG pMap;

   /**********************************************/
   /* Program ROM MAP register.                  */
   /**********************************************/
   pMap       = (LPREG) PCI_ROM_MAP_BASE  + bank;

   *pMap = ((((~((banksize>>16)-1)) << PCI_ROM_MAP_SIZE_SHIFT) & PCI_ROM_MAP_SIZE_MASK) |
                ((bankbase>>16) << PCI_ROM_MAP_OFFSET_SHIFT));

   /*printf("bank%d, bankbase=0x%x, banksize=%x\n", bank, bankbase, banksize );*/
}

/********************************************************************/
/*  CalcFreqFromFracPart                                            */
/*                                                                  */
/*  DESCRIPTION:                                                    */
/*      Calculate ( freq * pll_frac ) / ( 2 ^ frac_len )            */
/*      Note that ( freq * pll_frac ) is much bigger                */
/*      than unsigned 0xffffffff.  The algorithm used may result    */
/*      up to 3 less than what should be                            */
/*                                                                  */
/*  PARAMETERS:                                                     */
/*      See description                                             */
/*                                                                  */
/*  RETURNS:                                                        */
/*      result described above                                      */
/********************************************************************/
static ulong CalcFreqFromFracPart(
                    ulong uFreq,
                    ulong uPLLDivider,
                    ulong uDividerWidth)
{
    unsigned long long uCalc;

    /* In the following calculations, the frac value read from the*/
    /* PLL multiplier must be first converted to a floating       */
    /* point value, then divided by the number of bits used to    */
    /* represent the frac value (2^16) represented as a floating  */
    /* point value. This will cause the fixed-point integer       */
    /* representation of a fractional value to be converted to a  */
    /* floating-point fractional value.                           */

    uCalc = ((unsigned long long)uFreq * (unsigned long long)(uPLLDivider & ((1 << uDividerWidth)-1)))/(unsigned long long)(1 << uDividerWidth);
 
    return((ulong)uCalc);
}

/******************************************************************************/
/*  CalcClkFreqAndPeriodForIOPLL                                              */
/*                                                                            */
/*  DESCRIPTION:                                                              */
/*      Calculate clock frequency and period for a IO PLL.                    */
/*                                                                            */
/*  PARAMETERS:                                                               */
/*      xtal_freq  - Xtal frequency in Hz                                     */
/*                                                                            */
/*  RETURNS:                                                                  */
/*      frequency  - Returned clock frequency in Hz                           */
/*      period     - Returned clock period in 100ns increments                */
/******************************************************************************/
void CalcClkFreqAndPeriodForIOPLL (ulong *frequency,
                                   ulong *period,
                                   ulong xtal_freq )
{
    ulong pll_int;
    ulong pll_frac;
    ulong pll_div;
    ulong pll_prescale = 0;
    ulong clk_freq;
    ulong remainder;
    ulong frac_shift = 0;
    ulong clk_period;
    ulong pll_sel =0;

    *frequency = 0;
    *period    = 0;

    /* For Pecos, there is no dedicated I/O PLL, any of the 9 PLLs can source it.
     * We need to derive this from the PLL_SEL bits for the HSX0 clock*/
    pll_sel  = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_SEL_MASK);

    /* Handle the Verve PLLs in a common fashion */
    /* All the INTFRAC and CTRL regs for the Verve PLLs start at offset 0 of 
     * PLL_BASE with 0x4 increments. We derive the corresponding INTFRAC and
     * CTRL reg offsets from the corresponding PLL_SEL value
     *
     * INT_MASK = 0x7E000000 
     * FRAC_MASK = 0x01FFFFFF 
     * PREDIV_MASK = 0x000F0000 
     */
    /* WGS: converted switch statement to if statement to avoid the compiler to branch to __ARM_switch8 function which
       has read byte instructions( ldrccb and ldrcsb ). Note codeldr is executing from ROM, and Pecos RevA cannot
       do a byte read in ROM space correctly due to a chip bug. */   
    if( (pll_sel == MPG0PLL) || (pll_sel == MPG1PLL) || 
        (pll_sel == HDPLL)    || (pll_sel == AUDPLL) )
    {
       pll_int = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), PLL_MPG0_INTFRAC_INT_MASK);
       pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * pll_sel * 0x4)), PLL_MPG0_INTFRAC_FRAC_MASK);
       pll_div = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK);
       pll_prescale = CNXT_GET_VAL((PLL_BASE + 4 + (2 * pll_sel * 0x4)), PLL_MPG0_CTRL_PRE_DIV_MASK);
    }
    else if( (pll_sel == PLL0 ) || (pll_sel == PLL1) || 
             (pll_sel == PLL2) )
    {
       pll_int = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_INTFRAC_INT_MASK);
       pll_frac = CNXT_GET_VAL((PLL_BASE + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_INTFRAC_FRAC_MASK);
       pll_div = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK);
       pll_prescale = CNXT_GET_VAL((PLL_BASE + 4 + (2 * (pll_sel - 1) * 0x4)), PLL_MPG0_CTRL_PRE_DIV_MASK);
    }
    else if ( pll_sel == FENRUSPLL )
    {
       pll_int  = CNXT_GET_VAL(PLL_FENRUS_CTRL_REG, PLL_FENRUS_CTRL_LOOP_DIV_MASK);
       pll_frac = 0;
       pll_div  = CNXT_GET_VAL(DIV_MUX_CTRL10_REG, DIV_MUX_CTRL10_HSX0_PLL_DIV_MASK);
       xtal_freq    = CRYSTAL_FREQUENCY;
       pll_prescale = 1;
    }
    else
    {
       /* should not get here */
       return;
    }

    pll_div = pll_div * pll_prescale;

    /* 
     * calculate clk_freq = xtal_freq*(pll_int+pll_frac/2^n)/pll_div
     * note that algorithm used here may result up to 4Hz less than what it should be
     */
    clk_freq  = xtal_freq / pll_div;
    remainder = xtal_freq % pll_div;

    if( pll_sel == FENRUSPLL)
    {
       clk_freq = (clk_freq /pll_div) * pll_int;
    }
    else
    {
       clk_freq = CalcFreqFromFracPart ( clk_freq, pll_frac, (25-frac_shift))
                    + (clk_freq * pll_int) + (( remainder * pll_int ) / pll_div); 
    }     

    /*
     * Calculate clk_period = (10^11)/clk_freq
     * note that the algorithm will be broken if clk_freq is higher
     * than 429.5MHz.  it should not happen in the predictable future.
     */
    clk_period  = ( 1000000000 / clk_freq ) * 100;
    remainder   = ( 1000000000 % clk_freq ) * 10;

    clk_period += ( remainder / clk_freq ) * 10;
    remainder   = ( remainder % clk_freq ) * 10;

    clk_period += remainder / clk_freq;

    /*
     * Return the values
     */
    *frequency = clk_freq;
    *period    = clk_period;

}

#endif

