/****************************************************************************/
/*                   CONEXANT PROPRIETARY AND CONFIDENTIAL                  */
/*                       SOFTWARE FILE/MODULE HEADER                        */
/*                   Conexant Systems Inc. (c) 1998 - 2007                  */
/*                            All Rights Reserved                           */
/****************************************************************************/
/*
 * Filename:       mgpio.c
 *
 *
 * Description:    Mini GPIO access functions
 *
 *
 * Author:         Dave Wilson
 *
 ****************************************************************************/
/* $Id: mgpio.c 55536 2008-09-12 17:08:25Z swartzwg $
 ****************************************************************************/

/*****************/
/* Include Files */
/*****************/
#include "stbcfg.h"
#include "basetype.h"
#include "status_codes.h"
#include "mgpio.h"

/* 
 * ToDo: Add code to detect if RTOS is up and running.  If it is not, then
 * bail out on any GPIO manipulations that are I2C.
 */
#if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IIC)
   #error "GPIO Support for IIC Not Fully Implemented"
#endif

/******************************************/
/* Local Definitions and Global Variables */
/******************************************/
static u_int32 split_gpio_int(u_int32 iInt, u_int32 *uBank, u_int32 *uBitNum);
static void    set_gpio_output_level(unsigned int uGpioNum, bool bHigh);
static int     get_gpio_input(unsigned int  GPIO_bit);
static void    set_gpio_output_off(unsigned int GPIO_bit);
static void    set_gpio_int_edge(unsigned int  GPIO_bit, int  polarity);

#if RTOS == NOOS
extern HW_DWORD CNXT_PIO_EXP_REG_SHADOW;
#else
/* Shadow variable for write-only ISA extender */
#if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO) && (CNXT_PIO_EXP_REG != NOT_PRESENT)
HW_DWORD CNXT_PIO_EXP_REG_SHADOW = DEFAULT_PIO_EXP_REG_VALUE;
#endif
#endif

/********************************************************************/
/*  FUNCTION:    cnxt_gpio_set_output_level                         */
/*                                                                  */
/*  PARAMETERS:  uGPIO    - General GPIO identifier for the pin     */
/*                          whose logic level is to be set.         */
/*               bOn      - TRUE to set logic level 1, FALSE for 0  */
/*                                                                  */
/*  DESCRIPTION: This function sets the state of an individual GPIO */
/*               output to logical 1 or 0. The output level is      */
/*               determined based upon whether the GPIO is marked   */
/*               GPIO_NEGATIVE_POLARITY or GPIO_POSITIVE_POLARITY.  */
/*                                                                  */
/*  RETURN VALUES:                                                  */
/*               CNXT_STATUS_OK,                                    */
/*               CNXT_STATUS_INVALID_DEVICE,                        */
/*               CNXT_STATUS_INVALID_NUMBER,                        */
/*               CNXT_STATUS_INVALID_TYPE                           */
/*                                                                  */
/*  CONTEXT:     Interrupt safe if the GPIO passed is NOT connected */
/*               to an I2C GPIO extender on the target board.       */
/*                                                                  */
/********************************************************************/
CNXT_STATUS cnxt_gpio_set_output_level(u_int32 uGPIO, bool bOn)
{
  u_int32          uDevice;
  u_int16          uBit;
  CNXT_STATUS eReturn = CNXT_STATUS_OK;
  
  /* GPIO must be an output */
  if((uGPIO & GPIO_PIN_INPUT_MASK) == GPIO_PIN_IS_INPUT)
  {
    eReturn = CNXT_STATUS_INVALID_TYPE;
  }
  else
  {  
    /* Depending upon the device type, do the right thing */
    uDevice = uGPIO & GPIO_MASK_DEVICE_ID;
    uBit    = uGPIO & GPIO_MASK_PIN_NUMBER;
    
    switch (uDevice)
    {
      /* A normal, internal GPIO pin */
      case GPIO_DEVICE_ID_INTERNAL:
        set_gpio_output_level(uBit, 
                              ((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY) ? bOn : !bOn);
        break;
      
      #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IIC)
      /* First I2C GPIO extender */  
      #if (I2C_BUS_NIM_EXT != I2C_BUS_NONE)
      case GPIO_DEVICE_ID_I2C_1:
        if(uBit < 8)
        {
          if((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY)
            write_gpio_extender ( (u_int8)( 1 << uBit ), bOn ? (u_int8)( 1 << uBit ) : (u_int8)0 );
          else  
            write_gpio_extender ( (u_int8)( 1 << uBit ), !bOn ? (u_int8)( 1 << uBit ) : (u_int8)0 );
        }  
        else
          eReturn = CNXT_STATUS_INVALID_NUMBER;  
        break;
      #endif  
        
      /* Second I2C GPIO extender */  
      #if (I2C_BUS_GPIO2_EXT != I2C_BUS_NONE)
      case GPIO_DEVICE_ID_I2C_2:
        if(uBit < 8)
        {                                 
          if((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY)
            write_second_gpio_extender ( (u_int8)( 1 << uBit ), bOn ? (u_int8)( 1 << uBit ) : (u_int8)0);
          else  
            write_second_gpio_extender ( (u_int8)( 1 << uBit ), !bOn ? (u_int8)( 1 << uBit ) : (u_int8)0);
        }  
        else
          eReturn = CNXT_STATUS_INVALID_NUMBER;  
        break;
      #endif  
      #endif
        
      #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO)
      /* ISA-connected GPIO extender */
      #if (CNXT_PIO_EXP_REG != NOT_PRESENT)
      case GPIO_DEVICE_ID_ISA_1:
        if( (((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY) && bOn) ||
            (((uGPIO & GPIO_POLARITY_MASK) == GPIO_NEGATIVE_POLARITY) && !bOn))
          CNXT_PIO_EXP_REG_SHADOW |= (u_int16)( 1<<(uGPIO & GPIO_MASK_PIN_NUMBER));
        else  
          CNXT_PIO_EXP_REG_SHADOW &= (u_int16)~( 1<<(uGPIO & GPIO_MASK_PIN_NUMBER));
        *((LPREG)CNXT_PIO_EXP_REG) = CNXT_PIO_EXP_REG_SHADOW;
        break;
      #endif  
        
      #endif
        
      /* An invalid device code was passed in the GPIO description */  
      default:
        eReturn = CNXT_STATUS_INVALID_DEVICE;
        break;
    }
  } 
  
  return(eReturn);  
}


/********************************************************************/
/*  FUNCTION:    cnxt_gpio_get_level                                */
/*                                                                  */
/*  PARAMETERS:  uGPIO    - General GPIO identifier for the pin     */
/*                          whose logic level is to be set.         */
/*               pbOn     - Pointer to storage for returned logic   */
/*                          level.                                  */
/*                                                                  */
/*  DESCRIPTION: This function returns the current value of a GPIO  */
/*               pin. If it is an output, the last value written    */
/*               will be returned, else the value will indicate the */
/*               level on an input pin. The logic level is written  */
/*               to the address pointed to by pbOn.                 */
/*                                                                  */
/*               Note that the returned logic level takes into      */
/*               account the state of the GPIO polarity bit in the  */
/*               generic GPIO descriptor passed to the function.    */
/*                                                                  */
/*  RETURN VALUES:                                                  */
/*               CNXT_STATUS_OK,                                    */
/*               CNXT_STATUS_INVALID_DEVICE,                        */
/*               CNXT_STATUS_INVALID_NUMBER,                        */
/*               CNXT_STATUS_INVALID_TYPE,                          */
/*               CNXT_STATUS_BAD_PTR                                */
/*                                                                  */
/*  CONTEXT:     Interrupt safe if the GPIO passed is NOT connected */
/*               to an I2C GPIO extender on the target board.       */
/*                                                                  */
/********************************************************************/
CNXT_STATUS cnxt_gpio_get_level(u_int32 uGPIO, bool *pbOn)
{
  u_int32 uDevice;
  u_int16          uBit;
  int              iLevel;
  CNXT_STATUS eReturn = CNXT_STATUS_OK;
  #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IIC)
  u_int8  cRead;
  #endif
  #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO)
  u_int32 uValue;
  #endif
  
  if(!pbOn)
    return(CNXT_STATUS_BAD_PTR);
    
  /* Depending upon the device type, do the right thing */
  uDevice = uGPIO & GPIO_MASK_DEVICE_ID;
  uBit    = uGPIO & GPIO_MASK_PIN_NUMBER;
  
  switch (uDevice)
  {
    /* A normal, internal GPIO pin */
    case GPIO_DEVICE_ID_INTERNAL:
      iLevel = get_gpio_input(uBit);
      *pbOn =  (((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY) ? (iLevel != 0) : (iLevel == 0));
      break;
    
    #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IIC)
    /* First I2C GPIO extender */  
    #if (I2C_BUS_NIM_EXT != I2C_BUS_NONE)
    case GPIO_DEVICE_ID_I2C_1:
      if(uBit < 8)
      {
        cRead = read_gpio_extender();
        cRead &= (u_int8)( 1 << uBit );
        *pbOn =  (((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY) ? (cRead != 0) : (cRead == 0));
      }  
      else
        eReturn = CNXT_STATUS_INVALID_NUMBER;  
      break;
    #endif  
      
    /* Second I2C GPIO extender */  
    #if (I2C_BUS_GPIO2_EXT != I2C_BUS_NONE)
    case GPIO_DEVICE_ID_I2C_2:
      if(uBit < 8)
      {                                 
        cRead = read_second_gpio_extender();
        cRead &= (u_int8)( 1 << uBit );
        *pbOn =  (((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY) ? (cRead != 0) : (cRead == 0));
      }  
      else
        eReturn = CNXT_STATUS_INVALID_NUMBER;  
      break;
    #endif  
    #endif
      
    #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO) && (CNXT_PIO_EXP_REG != NOT_PRESENT)
    /* ISA-connected GPIO extender - note this is NOT read-modify-write so we */
    /* must use a shadow copy to ensure that we update the extender correctly!*/
    case GPIO_DEVICE_ID_ISA_1:
      uValue = CNXT_PIO_EXP_REG_SHADOW & (1<<(uGPIO & GPIO_MASK_PIN_NUMBER));
      *pbOn =  (((uGPIO & GPIO_POLARITY_MASK) == GPIO_POSITIVE_POLARITY) ? (uValue != 0) : (uValue == 0));
      break;
    #endif
      
    /* An invalid device code was passed in the GPIO description */  
    default:
      eReturn = CNXT_STATUS_INVALID_DEVICE;
      break;
  }

  return(eReturn);  
}

/********************************************************************/
/*  FUNCTION:    cnxt_gpio_set_input                                */
/*                                                                  */
/*  PARAMETERS:  uGPIO    - General GPIO identifier for the pin     */
/*                          which is to be set as an input.         */
/*                                                                  */
/*  DESCRIPTION: This function sets the supplied GPIO pin as an     */
/*               input assuming this is possible.                   */
/*                                                                  */
/*  RETURN VALUES:                                                  */
/*               CNXT_STATUS_OK,                                    */
/*               CNXT_STATUS_INVALID_DEVICE,                        */
/*               CNXT_STATUS_INVALID_NUMBER,                        */
/*               CNXT_STATUS_INVALID_TYPE,                          */
/*               CNXT_STATUS_INVALID_STATE                          */
/*                                                                  */
/*  CONTEXT:     Interrupt safe if the GPIO passed is NOT connected */
/*               to an I2C GPIO extender on the target board.       */
/*                                                                  */
/********************************************************************/
CNXT_STATUS cnxt_gpio_set_input(u_int32 uGPIO)
{
  u_int32          uDevice;
  u_int16          uBit;
  CNXT_STATUS eReturn = CNXT_STATUS_OK;
  
  /* Depending upon the device type, do the right thing */
  uDevice = uGPIO & GPIO_MASK_DEVICE_ID;
  uBit    = uGPIO & GPIO_MASK_PIN_NUMBER;
  
  switch (uDevice)
  {
    /* A normal, internal GPIO pin */
    case GPIO_DEVICE_ID_INTERNAL:
      set_gpio_output_off(uBit);
      break;
    
    #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IIC)
    /* First I2C GPIO extender - cannot act as an input */  
    #if (I2C_BUS_NIM_EXT != I2C_BUS_NONE)
    case GPIO_DEVICE_ID_I2C_1:
      eReturn = CNXT_STATUS_INVALID_STATE;  
      break;
    #endif  
      
    /* Second I2C GPIO extender - cannot act as an input */  
    #if (I2C_BUS_GPIO2_EXT != I2C_BUS_NONE)
    case GPIO_DEVICE_ID_I2C_2:
      eReturn = CNXT_STATUS_INVALID_STATE;  
      break;
    #endif  
    #endif
      
    #if (CNXT_PIO_EXP_TYPE == GPIO_EXTEND_IO)
    /* ISA-connected GPIO extender - cannot act as an input */
    #if (CNXT_PIO_EXP_REG != NOT_PRESENT)
    case GPIO_DEVICE_ID_ISA_1:
      eReturn = CNXT_STATUS_INVALID_STATE;  
      break;
    #endif  
    #endif
      
    /* An invalid device code was passed in the GPIO description */  
    default:
      eReturn = CNXT_STATUS_INVALID_DEVICE;
      break;
  }
  
  return(eReturn);
}


/************************/
/************************/
/** INTERNAL FUNCTIONS **/
/************************/
/************************/

/***************************************************************/
/* Set a given internal GPIO pin output high or low.           */
/*                                                             */
/* Note that this function is for internal GPIOs ONLY and does */
/* not handle any GPIO on an external expander. This is by     */
/* design while we transition to the use of general purpose    */
/* GPIO pin descriptions in higher layers of code.             */
/***************************************************************/
static void set_gpio_output_level(unsigned int uGpioNum, bool bHigh)
{
   unsigned long Bank, BitPos;

   /* Convert from a linear, internal GPIO number to a bank */
   /* and bit position.                                     */
   split_gpio_int(uGpioNum, &Bank, &BitPos);

   if(bHigh)
   {
     DRIVE_GPIO_HIGH_BANK(Bank, BitPos);
   }
   else
   {
     DRIVE_GPIO_LOW_BANK(Bank, BitPos);
   }
}

/*************************************************************************************/
/* GPIO_bit is a number from 0 to 95                                                 */
/*************************************************************************************/
static int get_gpio_input(unsigned int  GPIO_bit)
{
  unsigned long Bank, BitPos;

  /* find which bit in which of 3 registers */
  split_gpio_int(GPIO_bit, &Bank, &BitPos);
  return GET_GPIO_PIN_BANK(Bank, BitPos);
}

/********************************************************************/
/*  FUNCTION:    set_gpio_output_off                                */
/*                                                                  */
/*  PARAMETERS:  uGpioNum - Linear GPIO number to make an input     */
/*                                                                  */
/*  DESCRIPTION: Set a given GPIO as an input. This function        */
/*               maps a linear GPIO number to the relevant bank     */
/*               and pin then writes the correct register.          */
/*                                                                  */
/*  RETURNS:     Nothing                                            */
/********************************************************************/
static void set_gpio_output_off(unsigned int GPIO_bit)
{
  unsigned long Bank, BitPos;

  split_gpio_int(GPIO_bit, &Bank, &BitPos);
  MAKE_GPIO_INPUT_BANK(Bank, BitPos);
}

/****************************************************************/
/* Split a "linear" GPIO number into bank number and bit number */
/****************************************************************/
static u_int32 split_gpio_int(u_int32 iInt, u_int32 *uBank, u_int32 *uBitNum)
{
   *uBitNum = iInt & 31;
   *uBank   = iInt >> 5;

   return(TRUE);
}

/****************************************************************************
 * Modifications:
 * $Log$
 * 
 ****************************************************************************/
 


